import pygame

import driver
import data
from playing import PlayingGameState

class Button(driver.State):
    def __init__(self, text, font):
        self.surface_text = font.render(text, False, (255, 255, 255))
        self.w = self.surface_text.get_width() + 4
        self.h = self.surface_text.get_height() + 4
        self.x = 0
        self.y = 0
    
    @property
    def bottom(self):
        return self.y + self.h

    def clicked(self, pos):
        mx, my = pos
        if (mx >= self.x and mx < self.x + self.w and
            my >= self.y and my < self.y + self.h):
            return True
        return False
    
    def draw(self, screen):
        screen.fill((128, 128, 255), (self.x, self.y, self.w, self.h))
        screen.fill((0, 0, 255), (self.x+1, self.y+1, self.w-2, self.h-2))
        screen.blit(self.surface_text, (self.x + 2, self.y + 2))
        
class TitleState(driver.State):
    
    def __init__(self, driver):
        super(TitleState, self).__init__(driver)
        self.surface_title = pygame.image.load(
            data.filepath("title.png")).convert()
        self.font = pygame.font.Font(None, 24)
        self.easy_game = Button("New Game (normal)", self.font)
        self.easy_game.x = 10
        self.easy_game.y = self.driver.h / 2
        
        self.hard_game = Button("New Game (hard)", self.font)
        self.hard_game.x = 10
        self.hard_game.y = self.easy_game.bottom + 10
        
        self.instructions = Button("Instructions", self.font)
        self.instructions.x = 10
        self.instructions.y = self.hard_game.bottom + 10
        
        self.quit = Button("Quit", self.font)
        self.quit.x = 10
        self.quit.y = self.instructions.bottom + 10
        
        self.drawables = [
            self.easy_game,
            self.hard_game,
            self.instructions,
            self.quit,
        ]

    def draw(self, screen):
        screen.blit(self.surface_title, (0, 0))
        for d in self.drawables:
            d.draw(screen)

    def handle_click(self, button):
        if button == self.quit:
            self.driver.pop()
        elif button == self.easy_game:
            self.start_game()
        elif button == self.hard_game:
            self.start_game(True)
        elif button == self.instructions:
            self.driver.push(InstructionState(self.driver))

    def mouse(self, pos, button, pressed):
        if not pressed:
            for d in self.drawables:
                if d.clicked(pos):
                    self.handle_click(d)

    def start_game(self, hard=False):
        pgs = PlayingGameState(self.driver, hard)
        self.driver.push(pgs)

class InstructionState(driver.State):
    def __init__(self, driver):
        super(InstructionState, self).__init__(driver)
        
        self.surface_instructions = pygame.image.load(
            data.filepath("instructions.png")).convert()
        self.font = pygame.font.Font(None, 24)
        self.done = Button("Done", self.font)
        
        self.done.x = 10
        self.done.y = self.driver.h - 100
    
    def draw(self, screen):
        screen.blit(self.surface_instructions, (0, 0))
        self.done.draw(screen)

    def mouse(self, pos, button, pressed):
        if not pressed and self.done.clicked(pos):
            self.driver.pop()
